<?php

class ActiveRecord extends CActiveRecord
{
/**
	 * Based on ActiveRecord delete() but instead of actually deleting it sets an attribute to deleted=true.
	 * Others have chosen to implement this as a behavior; this should be standard AR practice, but in order
	 * to maintain backwards compatibility with a few models which use delete() to actually delete models it must not be overridden (delete).
	 *
	 * Performs a logical delete.
	 **/
	public function softDelete()
	{
		if(!$this->getIsNewRecord())
		{
			$attributeName = $this->getDeletedAttributeName();
			if( empty($attributeName) )
				throw new Exception('The active record cannot be soft deleted without an appropriately named attribute (deleted, visible, active).');

			Yii::trace(get_class($this).'.delete()','system.db.ar.CActiveRecord');
			if($this->beforeDelete())
			{
				// value is 'deleted'
				if( $attributeName == 'deleted' )
					$value = 1;
				// value is 'visible', or 'active' 
				else
					$value = 0;
				
				$result = $this->saveAttributes(array($attributeName=>$value)) > 0;
				$this->afterDelete();
				return $result;
			}
			else
				return false;
		}
		else
			throw new CDbException(Yii::t('yii','The active record cannot be deleted because it is new.'));
	}

	/**
	 * Used by softDelete() to determine the name of the attribute which is to be updated
	 * in order to achieve a logical delete.
	 **/
	private function getDeletedAttributeName()
	{
		foreach( array('deleted','visible','active') as $attr )
		{
			if( $this->hasAttribute($attr) )
				return $attr;
		}	
		return false;
	}

}
?>